@@ -3,6 +3,7 @@ mod datafusion {
     use std::sync::Arc;
 
     use arrow::array::*;
+    use datafusion::datasource::TableProvider;
     use datafusion::error::Result;
     use datafusion::execution::context::ExecutionContext;
 
@@ -50,4 +51,27 @@ mod datafusion {
 
         Ok(())
     }
+
+    #[tokio::test]
+    async fn test_datafusion_stats() -> Result<()> {
+        let table = deltalake::open_table("./tests/data/delta-0.8.0")
+            .await
+            .unwrap();
+        let statistics = table.statistics();
+
+        assert_eq!(statistics.num_rows, Some(4),);
+
+        assert_eq!(statistics.total_byte_size, Some(440 + 440));
+
+        assert_eq!(
+            statistics
+                .column_statistics
+                .unwrap()
+                .iter()
+                .map(|x| x.null_count)
+                .collect::<Vec<Option<usize>>>(),
+            vec![Some(0)],
+        );
+        Ok(())
+    }
 }
